#-*-coding:utf-8 -*-
#author:Mengmeng.liu

"""calculate the number of pixels located in three color domain"""
from PIL import Image
import numpy as np
import cv2 as cv

"""the boundary of domain"""
def reference():

    x530 = 0.1547
    y530 = 0.8059

    x570 = 0.4441
    y570 = 0.5547

    x580 = 0.5125
    y580 = 0.4866

    slope_530 = (y530 - 0.33) / (x530 - 0.33)
    slope_570 = (y570 - 0.33) / (x570 - 0.33)
    slope_620 = (y580 - 0.33) / (x580 - 0.33)
    return slope_530,slope_570,slope_620

a,b,c = reference()

"""calculate the value of H in HSB"""
def colorblue(r,g,b,count_blue):
    if (r == max(r,g,b)) & (g >= b):
        h = (int(g) - int(b)) / (int(r) - int(b)) * 60
    elif (r == max(r,g,b)) & (g < b):
        h = (int(g) - int(b)) / (int(r) - int(g)) * 60+ 360
    elif (g == max(r,g,b)):
        h = (float(b) - float(r))/(float(g) - float(min(r,g,b))) * 60 + 120
    elif (b == max(r,g,b)):
        h = (int(r) - int(g)) / (int(b) - int(min(r,g,b))) * 60 + 240
    if h > 120: # the H threshold
        count_blue += 1
    return count_blue
"""calculate the value of S in HSB"""
def colorhsb_s(r,g,b):
    max_color = float(max(r,g,b))
    min_color = float(min(r,g,b))
    saturace = 1 - min_color/max_color
    return saturace

"""calculate the value of x,y,z in CIE-map"""
def color(r,g,blue,count_green,count_yellow,count_red,a,b,c):
    r = float(r)
    g = float(g)
    blue = float(blue)
    X = r * 0.49 + 0.31 * g + 0.2 * blue
    Y = r * 0.117 + 0.812 * g + 0.01 * blue
    Z = r * 0 + 0.01 * g + 0.99 * blue
    x = round(X/(X + Y + Z),2)
    y = round(Y/(X + Y + Z),2)
    z = round(Z/(X + Y + Z),2)
    if (x != 0.33):
        slope_real = (y - 0.33) / (x - 0.33)
        if (slope_real < a)|(slope_real > b):
            count_green += 1
        elif (slope_real < b)&(slope_real > c):
            count_yellow += 1
        else:
            if x > 0.33:
                count_red += 1
    return count_green,count_yellow,count_red
"""this is the main function, calculate the output value in this function"""
def readpixel(image,*kwds):
    G = []
    H = []
    Intensity = []
    LOcation = []
    color_r = []
    color_b = []
    color_g = []
    a,b,d = reference()
    # output the TL and TU
    if 'step' in kwds:
        imagedata = image.flatten()
        imagedata = np.sort(imagedata)
        length = int(len(imagedata) * kwds[1])
        thershold = imagedata[length]
        return thershold
    else:
        # output the domain location of spot
        c = kwds[0]
        count_blue = 0
        for i in range(len(c['x'])):
            for j in c['y'][i]:
                loc = image[j,c['x'][i]]
                intensity = round((float(loc[2]) * 0.114 + float(loc[1]) * 0.5876 + float(loc[0]) * 0.299),2)
                color_r.append(loc[2])
                color_g.append(loc[1])
                color_b.append(loc[0])
                Intensity.append(intensity)
                LOcation.append([j,c['x'][i]])
                saturace = 0
                Max_intensity = max(Intensity)
                if (max(loc) != min(loc)):
                    count_blue = colorblue(loc[2],loc[1],loc[0],count_blue)
                    saturace = max(colorhsb_s(loc[2],loc[1],loc[0]),saturace)
                else:
                    continue
                if (intensity <= 240): # the I threshold
                    saturace = max(colorhsb_s(loc[2],loc[1],loc[0]),saturace)
        Max_intensity = max(Intensity)

        if len(Intensity ) > 5:
            if count_blue > 4:
                pointcolor =  'blue point'
                return pointcolor
        if len(Intensity )<= 5:
            if count_blue > 1:
                pointcolor = 'blue point'
                return pointcolor
        # find out the brightyellow spot
        if ((Max_intensity < 220)&(saturace < 0.33)): # the S threshold
            pointcolor = 'noise'
            return pointcolor
        elif (220 <= Max_intensity <= 240)&(saturace < 0.2):
            pointcolor = 'rednoise'
            return pointcolor

        Max_loc = Intensity.index(Max_intensity)
        location = LOcation[Max_loc]
        if (Max_intensity < 240):
            count_green = 0
            count_red = 0
            count_yellow = 0
            for i in range(len(color_b)):
                if (Intensity[i] > 80): # the TL
                    count_green, count_yellow, count_red = color(color_r[i],\
                    color_g[i],color_b[i],count_green,\
                    count_yellow,count_red,a,b,d)
            if (count_green == max(count_green,count_yellow,count_red)) & (count_green != 0):
                pointcolor = 'green'
            elif (count_yellow == max(count_green,count_yellow,count_red))& (count_yellow != 0):
                pointcolor = 'yellow'
            elif (count_red == max(count_green,count_yellow,count_red))& (count_red != 0):
                pointcolor = 'red'
            else:
                pointcolor = 'none'
        else:
            pointcolor = 'brightyellow'
        return [pointcolor,location]
